# query_vector_space.py
# Ambika Acharya and Emily Mallory
# 
# Parameters:
#    -d /path/to/output/directory
#    -t type of ranking (rank_drug or rank_rxn)
#    -s query input (either dm or reaction identifier)
#
######################################

import pickle as pkl
import sys
from numpy import linalg as LA
import numpy as np

#global dictionaries
reacts_to_smiles = {}
smiles_to_reacts = {}
smiles_to_indices = {}
dm_to_index = {}
db_diff_vecs = None
diff_index_to_smiles = None


def load_data(path):
	''' load necessary data files and diff vectors with specified directory path '''

	#load dictionary pickles
	smiles_to_reacts = pkl.load(open(path+"/smiles_to_reacts_maccs.p","rb"))
	smiles_indices = pkl.load(open(path+"/smiles_indices_MACCS.p"))

	#make a map of reactions to the pair of smiles codes of compounds in that reaction
	for line in smiles_to_reacts:
	  reacts_to_smiles[smiles_to_reacts[line].strip()]=line

	#compute difference vectors
	(db_diff_vecs, diff_index_to_smiles) = compute_diff_vectors()

	#make a map to smiles codes to indices in X_mol
	for line in diff_index_to_smiles:
	  smiles_to_indices[diff_index_to_smiles[line]] = line

	#make a map of DM query reaction to its index in the reaction vector space
	dm_file = open('drug_metabolite_data.tsv')
	for line in dm_file:
	  if line.startswith('react'):
	    continue
	  if line.strip().split('\t')[1] != drug_input: continue
	  react = line.strip().split('\t')[0]
	  dm_to_index[react] = smiles_to_indices[tuple(line.strip().split('\t')[3:5])]

def compute_diff_vectors():
	'''compute vector differences between reactions'''

    k_means_vecs = []
    diff_vecs = {}
    ind_count = 0
    not_in = 0
    tot = 0
    for smiles_pair in smiles_to_reacts:
            tot+=1
            if smiles_pair[0] not in smiles_indices or smiles_pair[1] not in smiles_indices:
                not_in+=1    
                continue
            fp_index_left =smiles_indices[smiles_pair[0]]
            fp_index_right =smiles_indices[smiles_pair[1]]
            left_vec = X_mol[fp_index_left,:]
            right_vec = X_mol[fp_index_right,:]
            left_vec = np.array([0.0 if np.isnan(i) else i for i in left_vec])
            right_vec = np.array([0.0 if np.isnan(i) else i for i in right_vec])
            diff = np.array([0.0 if np.isnan(i) else i for i in (np.array(right_vec) - np.array(left_vec))])
            k_means_vecs.append(diff)
            diff_vecs[ind_count] = smiles_pair
            ind_count +=1
    k_means_vecs = np.array(k_means_vecs)
    return (k_means_vecs,diff_vecs)

#query for top reaction matches to a given drug
def get_top_rxn_from_dm(quer_drug, output, num_rxn = 10):
	''' query top reactions (num_rxn, default 10) for query drug pair '''

	vec = db_diff_vecs[dm_to_index[quer_drug]]
	dists = {}
	for i in range(len(db_diff_vecs)):
		ex = db_diff_vecs[i]
		react_dist = np.sum(np.square(np.diff(ex-vec)))#euclidean
		dists[i] =((react_dist),(react_dist))
		#dists.append(np.dot(ex,vec)/(LA.norm(ex)*LA.norm(vec)))#cosine
	s_dists = sorted(dists.items(),key=lambda x:x[1][0])
	
	with open(output, "w") as o:
		count = 1 #number of reactions to return
		for ind,dist in s_dists:
			if count == num_rxn:
				break
			count+=1
			o.write("{0}\t{1}\n".format(smiles_to_reacts[diff_index_to_smiles[ind]], str(dist)))


def get_top_dm_from_query(quer_react, output, num_rxn = 10):
	''' query top dm pairs (num_rxn, default 10) for query reaction '''

	dists = {}
	ex = db_diff_vecs[smiles_to_indices[reacts_to_smiles[quer_react]]]
	print quer_react
	for q in dm_to_index:
		vec =db_diff_vecs[dm_to_index[q]]
  		dists[dm_to_index[q]] = (np.sum(np.square(np.diff(ex-vec))))#euclidean
  		#dists[dm_to_index[q]]=np.dot(ex,vec)/(LA.norm(ex)*LA.norm(vec))#cosine
	s_dists = sorted(dists.items(),key=lambda x:x[1])
	
	with open(output, "w") as o:
		count = 1
		for ind,dist in s_dists:
			if count == num_rxn:
				break
			count += 1
			o.write("{0}\t{1}\n".format(smiles_to_reacts[diff_index_to_smiles[ind]], str(dist)))


def get_arguments():
	'''
	Name: get_arguments
	Parses arguments from command line

	argparse arguments:
		-d: output directory
		-t: type of query ranking: rank_rxn, rank_drug
		-q: query input (either rxn identifier or dm identifier)
	'''

	parser = argparse.ArgumentParser()

	parser.add_argument('-d', action='store', dest='dir',
                    help='output directory')

	parser.add_argument('-t', action='store', dest='query_type',
                    help='type of query ranking (rank_rxn or rank_drug')

	parser.add_argument('-q', action='store', dest='query_input',
                    help='query input (reaction or drug-metabolite identifier')

	return parser.parse_args()

def main():

	# get command line arguments
	# -d output directory name
	# -t type of ranking
	# -q query input

	arguments = get_arguments()

	out_file = arguments.dir + "/output_" + arguments.query_type + "_" + arguments.query_input + ".tsv"

	if arguments.query_type == 'rank_rxn':

		#rank reactions for a query drug-metabolite pair
		get_top_rxn_from_dm(arguments.query_input, out_file, 10)

	elif arguments.query_type == 'rank_drug':

		#rank drug-metabolite pairs for a query reaction
		get_top_dm_from_query(arguments.query_input, out_file, 10)


if __name__ == '__main__':
	main()